@@ -16,89 +16,71 @@
 #ifndef CAML_CONFIG_H
 #define CAML_CONFIG_H
 
-#define ARCH_SIXTYFOUR
-#define SIZEOF_INT 4
-#define SIZEOF_LONG 8
-#define SIZEOF_PTR 8
-#define SIZEOF_SHORT 2
-#define SIZEOF_LONGLONG 8
-#define INT64_LITERAL(s) s ## LL
-#undef ARCH_BIG_ENDIAN
-#undef ARCH_ALIGN_DOUBLE
-#undef ARCH_ALIGN_INT64
-#define ASM_CFI_SUPPORTED
-#define PROFINFO_WIDTH 0
-#define CAML_WITH_CPLUGINS
-#define OCAML_OS_TYPE "Unix"
-#define OCAML_STDLIB_DIR "/usr/lib/ocaml"
-#define POSIX_SIGNALS
-#define HAS_C99_FLOAT_OPS
-#define HAS_GETRUSAGE
-#define HAS_TIMES
-#define HAS_SECURE_GETENV
-#define HAS_TERMCAP
-#define HAS_SOCKETS
-#define HAS_SOCKLEN_T
-#define HAS_INET_ATON
-#define HAS_IPV6
-#define HAS_STDINT_H
-#define HAS_UNISTD
-#define HAS_OFF_T
-#define HAS_DIRENT
-#define HAS_REWINDDIR
-#define HAS_LOCKF
-#define HAS_MKFIFO
-#define HAS_GETCWD
-#define HAS_GETWD
-#define HAS_GETPRIORITY
-#define HAS_UTIME
-#define HAS_UTIMES
-#define HAS_DUP2
-#define HAS_FCHMOD
-#define HAS_TRUNCATE
-#define HAS_SYS_SELECT_H
-#define HAS_SELECT
-#define HAS_NANOSLEEP
-#define HAS_SYMLINK
-#define HAS_WAITPID
-#define HAS_WAIT4
-#define HAS_GETGROUPS
-#define HAS_SETGROUPS
-#define HAS_INITGROUPS
-#define HAS_TERMIOS
-#define HAS_ASYNC_IO
-#define HAS_SETITIMER
-#define HAS_GETHOSTNAME
-#define HAS_UNAME
-#define HAS_GETTIMEOFDAY
-#define HAS_MKTIME
-#define HAS_SETSID
-#define HAS_PUTENV
-#define HAS_LOCALE
-#define SUPPORT_DYNAMIC_LINKING
-#define HAS_MMAP
-#define HAS_PWRITE
-#define HAS_NANOSECOND_STAT 1
-#define HAS_GETHOSTBYNAME_R 6
-#define HAS_GETHOSTBYADDR_R 8
-#define HAS_MKSTEMP
-#define HAS_NICE
-#define HAS_DUP3
-#define HAS_PIPE2
-#define HAS_ACCEPT4
-#define HAS_SIGWAIT
-#define HAS_LIBBFD
-#define HAS_HUGE_PAGES
-#define HUGE_PAGE_SIZE (4 * 1024 * 1024)
-
+/* CAML_NAME_SPACE was introduced in OCaml 3.08 to declare compatibility with
+   the newly caml_-prefixed names of C runtime functions and to disable the
+   definition of compatibility macros for the un-prefixed names. The
+   compatibility layer was removed in OCaml 5.0, so CAML_NAME_SPACE is the
+   default. */
 #ifndef CAML_NAME_SPACE
-#include "compatibility.h"
+#define CAML_NAME_SPACE
+#endif
+
+#include "m.h"
+
+/* If supported, tell gcc that we can use 32-bit code addresses for
+ * threaded code, unless we are compiled for a shared library (-fPIC option) */
+#ifdef HAS_ARCH_CODE32
+#ifndef __PIC__
+#  define ARCH_CODE32
+#endif /* __PIC__ */
+#endif /* HAS_ARCH_CODE32 */
+
+/* Microsoft introduced the LL integer literal suffix in Visual C++ .NET 2003 */
+#if defined(_MSC_VER) && _MSC_VER < 1400
+#define INT64_LITERAL(s) s ## i64
+#else
+#define INT64_LITERAL(s) s ## LL
+#endif
+
+#if defined(_MSC_VER) && !defined(__cplusplus)
+#define Caml_inline static __inline
+#else
+#define Caml_inline static inline
+#endif
+
+#include "s.h"
+
+#ifndef CAML_CONFIG_H_NO_TYPEDEFS
+
+#include <stddef.h>
+
+#if defined(HAS_LOCALE_H) || defined(HAS_XLOCALE_H)
+#define HAS_LOCALE
 #endif
 
 #ifdef HAS_STDINT_H
 #include <stdint.h>
 #endif
 
+/* Disable the mingw-w64 *printf shims */
+#if defined(CAML_INTERNALS) && defined(__MINGW32__)
+  /* Headers may have already included <_mingw.h>, so #undef if necessary. */
+  #ifdef __USE_MINGW_ANSI_STDIO
+    #undef __USE_MINGW_ANSI_STDIO
+  #endif
+  /* <stdio.h> must either be #include'd before this header or
+     __USE_MINGW_ANSI_STDIO needs to be 0 when <stdio.h> is processed. The final
+     effect will be the same - stdio.h will define snprintf and misc.h will make
+     snprintf a macro (referring to caml_snprintf). */
+  #define __USE_MINGW_ANSI_STDIO 0
+#endif
+
+#if defined(__MINGW32__) || (defined(_MSC_VER) && _MSC_VER < 1800)
+#define ARCH_SIZET_PRINTF_FORMAT "I"
+#else
+#define ARCH_SIZET_PRINTF_FORMAT "z"
+#endif
+
 /* Types for 32-bit integers, 64-bit integers, and
    native integers (as wide as a pointer type) */
 
@@ -120,18 +102,26 @@
 #endif
 #endif
 
-#ifndef ARCH_INT64_TYPE
-#if SIZEOF_LONGLONG == 8
-#define ARCH_INT64_TYPE long long
-#define ARCH_UINT64_TYPE unsigned long long
-#define ARCH_INT64_PRINTF_FORMAT "ll"
-#elif SIZEOF_LONG == 8
-#define ARCH_INT64_TYPE long
-#define ARCH_UINT64_TYPE unsigned long
-#define ARCH_INT64_PRINTF_FORMAT "l"
+#if defined(__MINGW32__) && !__USE_MINGW_ANSI_STDIO
+  #define ARCH_INT64_TYPE long long
+  #define ARCH_UINT64_TYPE unsigned long long
+  #define ARCH_INT64_PRINTF_FORMAT "I64"
+#elif defined(_MSC_VER)
+  #define ARCH_INT64_TYPE __int64
+  #define ARCH_UINT64_TYPE unsigned __int64
+  #define ARCH_INT64_PRINTF_FORMAT "I64"
 #else
-#error "No 64-bit integer type available"
-#endif
+  #if SIZEOF_LONG == 8
+    #define ARCH_INT64_TYPE long
+    #define ARCH_UINT64_TYPE unsigned long
+    #define ARCH_INT64_PRINTF_FORMAT "l"
+  #elif SIZEOF_LONGLONG == 8
+    #define ARCH_INT64_TYPE long long
+    #define ARCH_UINT64_TYPE unsigned long long
+    #define ARCH_INT64_PRINTF_FORMAT "ll"
+  #else
+    #error "No 64-bit integer type available"
+  #endif
 #endif
 
 #ifndef HAS_STDINT_H
@@ -146,6 +136,7 @@ typedef unsigned short uint16_t;
 #else
 #error "No 16-bit integer type available"
 #endif
+typedef unsigned char uint8_t;
 #endif
 
 #if SIZEOF_PTR == SIZEOF_LONG
@@ -167,6 +158,10 @@ typedef uint64_t uintnat;
 #error "No integer type available to represent pointers"
 #endif
 
+#define UINTNAT_MAX ((uintnat)-1)
+
+#endif /* CAML_CONFIG_H_NO_TYPEDEFS */
+
 /* Endianness of floats */
 
 /* ARCH_FLOAT_ENDIANNESS encodes the byte order of doubles as follows:
@@ -187,7 +182,7 @@ typedef uint64_t uintnat;
    as first-class values (GCC 2.x). */
 
 #if defined(__GNUC__) && __GNUC__ >= 2 && !defined(DEBUG) \
-    && !defined (SHRINKED_GNUC) && !defined(CAML_JIT)
+    && !defined (SHRINKED_GNUC)
 #define THREADED_CODE
 #endif
 
@@ -202,13 +197,27 @@ typedef uint64_t uintnat;
 #define Page_size (1 << Page_log)
 
 /* Initial size of stack (bytes). */
-#define Stack_size (4096 * sizeof(value))
+#ifdef DEBUG
+#define Stack_init_bsize (64 * sizeof(value))
+#else
+#define Stack_init_bsize (4096 * sizeof(value))
+#endif
 
 /* Minimum free size of stack (bytes); below that, it is reallocated. */
-#define Stack_threshold (256 * sizeof(value))
+#define Stack_threshold_words 32
+#define Stack_threshold (Stack_threshold_words * sizeof(value))
+
+/* Number of words used in the control structure at the start of a stack
+   (see fiber.h) */
+#ifdef ARCH_SIXTYFOUR
+#define Stack_ctx_words (6 + 1)
+#else
+#define Stack_ctx_words (6 + 2)
+#endif
 
 /* Default maximum size of the stack (words). */
-#define Max_stack_def (1024 * 1024)
+/* (1 Gib for 64-bit platforms, 512 Mib for 32-bit platforms) */
+#define Max_stack_def (128 * 1024 * 1024)
 
 
 /* Maximum size of a block allocated in the young generation (words). */
@@ -218,44 +227,21 @@ typedef uint64_t uintnat;
 
 
 /* Minimum size of the minor zone (words).
-   This must be at least [2 * Max_young_whsize]. */
-#define Minor_heap_min 4096
-
-/* Maximum size of the minor zone (words).
-   Must be greater than or equal to [Minor_heap_min].
-*/
-#define Minor_heap_max (1 << 28)
+   This must be at least [Max_young_wosize + 1]. */
+#define Minor_heap_min (Max_young_wosize + 1)
 
 /* Default size of the minor zone. (words)  */
 #define Minor_heap_def 262144
 
-
 /* Minimum size increment when growing the heap (words).
    Must be a multiple of [Page_size / sizeof (value)]. */
 #define Heap_chunk_min (15 * Page_size)
 
-/* Default size increment when growing the heap.
-   If this is <= 1000, it's a percentage of the current heap size.
-   If it is > 1000, it's a number of words. */
-#define Heap_chunk_def 15
-
-/* Default initial size of the major heap (words);
-   Must be a multiple of [Page_size / sizeof (value)]. */
-#define Init_heap_def (31 * Page_size)
-/* (about 512 kB for a 32-bit platform, 1 MB for a 64-bit platform.) */
-
 
 /* Default speed setting for the major GC.  The heap will grow until
    the dead objects and the free list represent this percentage of the
    total size of live objects. */
-#define Percent_free_def 80
-
-/* Default setting for the compacter: 500%
-   (i.e. trigger the compacter when 5/6 of the heap is free or garbage)
-   This can be set quite high because the overhead is over-estimated
-   when fragmentation occurs.
- */
-#define Max_percent_free_def 500
+#define Percent_free_def 120
 
 /* Default setting for the major GC slice smoothing window: 1
    (i.e. no smoothing)
@@ -265,4 +251,23 @@ typedef uint64_t uintnat;
 /* Maximum size of the major GC slice smoothing window. */
 #define Max_major_window 50
 
+/* Default setting for the ratio of custom garbage to major heap size.
+   Documented in gc.mli */
+#define Custom_major_ratio_def 44
+
+/* Default setting for the ratio of custom garbage to minor heap size.
+   Documented in gc.mli */
+#define Custom_minor_ratio_def 100
+
+/* Default setting for maximum size of custom objects counted as garbage
+   in the minor heap.
+   Documented in gc.mli */
+#define Custom_minor_max_bsz_def 8192
+
+/* Default allocation policy. */
+#define Allocation_policy_def caml_policy_best_fit
+
+/* Default size of runtime_events ringbuffers, in words, in powers of two */
+#define Default_runtime_events_log_wsize 16
+
 #endif /* CAML_CONFIG_H */
